
/*
 * $Id: ilog2dlog.cc,v 1.1.2.2 1999/04/05 19:09:08 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <iomanip.h>

#include "ILog.h"
#include "DLog.h"
#include "LogMeta.h"
#include "gadgets.h"

static
bool ilog2dlog(ILog &il, DLog &dl, int phaseTag) {
	Assert(phaseTag > 0);

	/*int pingTag = -1;
	if (const LogTagDescr *descr = il.tagDescr("i"))
		pingTag = descr->theTag;*/

	int cmntTag = -1;
	if (const LogTagDescr *descr = il.tagDescr("_comment"))
		cmntTag = descr->theTag;

	while (LogEntryPrefix prefix = il.begEntry()) {
		if (prefix.theTag == phaseTag) {
			if (dl.thePhaseRec.duration() < 0) {
				dl.thePhaseRec.load(il);
				dl.repRate(Ratio(
					dl.thePhaseRec.theXactLvl.decCnt(),
					dl.thePhaseRec.duration().secd()));
			} else {
				StatPhaseRec ph;
				ph.load(il);
				dl.thePhaseRec += ph;
				dl.repRate(Ratio(ph.theXactLvl.decCnt(), ph.duration().secd()));
			}

			// XXX: kludge, should maintain these in PhaseRec
			dl.theErrCnt = il.progress().theErrs.theCnt;
			dl.theNonErrCnt = il.progress().theXactComplCnt;
		} else
		if (prefix.theTag == cmntTag) {
			String s;
			il >> s;
			if (const char *h = strstr(s, "req_rate:")) {
				double req_rate = -1;
				Check(isNum(h+10, req_rate));
				dl.reqRate(req_rate);
			}
			
		}
		il.endEntry();
	}

	return il.stream() && il.stream()->eof();
}


// phase name must be already set in dl
bool ilog2dlog(const String &fname, DLog &dl) {

	ILog il;
	il.stream(fname, new ifstream(fname));
	il.loadHeader();

	dl.theSources.append(new String(fname));

	if (const LogTagDescr *descr = il.tagDescr("p-" + dl.thePhaseName))
		return ilog2dlog(il, dl, descr->theTag);

	cerr << here << "no `" << dl.thePhaseName << "' phase in `" << fname << "'" << endl;
	return false;
}
