
/*
 * $Id: Size.h,v 1.1.1.1 1999/01/27 08:40:45 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef X_SIZE_H
#define X_SIZE_H

#include <iostream.h>

#include "Assert.h"

// reduces size convertion problems

class Size {
	typedef int size_t;
	friend istream &operator >>(istream &is, Size &sz);
	friend ostream &operator <<(ostream &os, const Size &sz);

	public:
		static Size KB(int n) { return Size(n * (size_t)1024); }
		static Size MB(int n) { return Size::KB(n * (size_t)1024); }

	public:
		Size(size_t aSize): theSize(aSize) {}
		Size(): theSize(-1) {}

		Size operator ++() { return theSize++; }
		Size operator ++(int) { return theSize++; }
		Size operator += (Size sz) { return theSize += sz; }
		Size operator -= (Size sz) { return theSize -= sz; }
		Size operator *= (Size sz) { return theSize *= sz; }
		Size operator /= (Size sz) { return theSize /= sz; }
		Size operator %= (Size sz) { return theSize %= sz; }
		operator size_t() const { return theSize; } // XXX risky?

		Size &operator =(int bytes) { theSize = bytes; return *this; }

		int kb() const { return (theSize+1023)/1024; }

	protected:
		size_t theSize;
};

// XXX: add smart printing
inline istream &operator >>(istream &is, Size &sz) { return is >> sz.theSize; }
inline ostream &operator <<(ostream &os, const Size &sz) { return os << sz.theSize; }

#endif
